from django.db import models
from django.utils.translation import gettext as _


class Log(models.Model):
    phone = models.CharField(verbose_name=_('Phone number'), max_length=30)
    msg = models.TextField(verbose_name=_('Message'))
    status = models.CharField(verbose_name=_('Status'), max_length=10)
    status_code = models.IntegerField(verbose_name=_('Status code'))
    status_text = models.TextField(verbose_name=_('Status text'), blank=True, null=True)
    sms_id = models.CharField(verbose_name=_('SMS ID'), max_length=255, blank=True, null=True)
    cost = models.FloatField(verbose_name=_('Cost'), blank=True, null=True)
    created_at = models.DateTimeField(verbose_name=_('Created at'), auto_now_add=True)

    class Meta:
        verbose_name = _('Logs')
        verbose_name_plural = _('Log')
        ordering = ('-created_at', )



from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

from smsru.models import Log
from smsru.service import SmsRuApi
from smsru.signals import smsru_call_back_sms


@csrf_exempt
def sms_callback(request):
    if request.method == 'POST':
        data = [item[1][0].split('\n') for item in request.POST.lists() if 'data[' in item[0]]
        hash = request.POST.get('hash')
        api = SmsRuApi()
        if api.validate_callback(["\n".join(el) for el in data], hash):
            for data_item in data:
                item = Log.objects.filter(sms_id=data_item[1]).first()
                if item:
                    item.status_code = data_item[2]
                    item.save()
                    smsru_call_back_sms.send(item.__class__, instance=item, new_status=item.status_code)
    return HttpResponse(100)



from typing import List

from django.contrib import admin
from smsru.models import Log

from django.utils.translation import gettext as _

from smsru.service import SmsRuApi


def update_sms_status(model, request, queryset):
    api = SmsRuApi()
    for item in queryset:
        if item.sms_id:
            result, data = api.get_status(item.sms_id)
            if result:
                item.status = data['status']
                item.status_code = data['status_code']
                item.status_text = data['status_text']
                item.save()


update_sms_status.short_description = _("Update selected sms status")


@admin.register(Log)
class LogAdmin(admin.ModelAdmin):
    list_display = ['phone', 'sms_id', 'msg', 'status', 'status_text', 'created_at']
    date_hierarchy = 'created_at'
    actions = [update_sms_status]



from django.core.management import BaseCommand
from smsru.service import SmsRuApi


class Command(BaseCommand):
    help = 'Send SMS'

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            '--phone', dest='phone', default=None,
            help='Phone number',
        )
        parser.add_argument(
            '--msg', dest='msg', default=None,
            help='Message text',
        )

    def handle(self, *args, **options):
        phone = options.get('phone')
        msg = options.get('msg')

        api = SmsRuApi()
        result = api.send_one_sms(phone, msg)

        self.stdout.write(f"Result: {result}")



import hashlib
import re
import requests

from django.conf import settings
from smsru.models import Log


class SmsRuApi:
    __api_id = None
    __login = None
    __password = None
    __is_test = 0
    __from = None
    __partner_id = None
    __api_url = 'https://sms.ru/sms/'
    __api_url_my = 'https://sms.ru/my/'

    def __init__(self):
        self._get_settings()

    def _get_settings(self):
        setting_key = 'SMS_RU'
        sms_ru = getattr(settings, setting_key, None)
        if sms_ru is None:
            raise KeyError(f"Key {setting_key} not found in settings.py")

        self.__api_id = sms_ru.get('API_ID', None)
        if self.__api_id is None:
            self.__login = sms_ru.get('LOGIN', None)
            self.__password = sms_ru.get('PASSWORD', None)
            if self.__login is None or self.__password is None:
                raise KeyError(f"LOGIN and PASSWORD not found in {setting_key}")

        self.__from = sms_ru.get('SENDER', None)
        self.__is_test = int(sms_ru.get('TEST', False))

    @staticmethod
    def beautify_phone(phone: str) -> str:
        return re.sub(r"\D", "", phone)

    def __request(self, url: str, post_param: dict) -> dict:
        if self.__api_id:
            post_param['api_id'] = self.__api_id
        else:
            post_param['login'] = self.__login
            post_param['password'] = self.__password

        post_param['json'] = 1

        if self.__from:
            post_param['from'] = self.__from
        if self.__partner_id:
            post_param['partner_id'] = self.__partner_id

        post_param['test'] = self.__is_test

        response = requests.post(url, data=post_param)

        data = response.json()

        return data

    def _sms_request(self, post_param: dict) -> dict:
        mock_sms = {'status': False, 'status_code': None, 'sms_id': None}
        if 'to' in post_param:
            return_result = {k: mock_sms for k in post_param['to'].split(',')}
            phone_msg = {k: post_param['msg'] for k in post_param['to'].split(',')}
        else:
            return_result = {k: mock_sms for k, v in post_param['multi'].items()}
            phone_msg = post_param['multi']
            multi = post_param.pop('multi')
            for k, v in multi.items():
                post_param[f'multi[{k}]'] = v

        url = self.__api_url + 'send'
        data = self.__request(url, post_param)

        if isinstance(data, dict):
            if data['status'] == 'OK' and data['status_code'] == 100:
                for phone, result in data['sms'].items():
                    return_result[phone] = {
                        'status': result['status_code'] == 100,
                        'status_code': result['status_code'],
                        'sms_id': result.get('sms_id', None),
                        'status_text': result.get('status_text', None)
                    }

                    itm = Log(
                        phone=phone,
                        status=result["status"],
                        msg=phone_msg.get(phone, None),
                        status_code=result["status_code"],
                        status_text=result.get("status_text", None),
                        sms_id=result.get("sms_id", None),
                        cost=result.get("cost", None),
                    )
                    itm.save()

        return return_result

    def send_one_sms(self, phone: str, msg: str) -> dict:
        phone_beautify = self.beautify_phone(phone)
        if phone_beautify:
            post_param = {
                'to': phone_beautify,
                'msg': msg
            }
            return self._sms_request(post_param)
        else:
            raise Exception(f'Bad phone number {phone_beautify}')

    def get_status(self, sms_id: str):
        post_param = {
            'sms_id': sms_id
        }
        url = self.__api_url + 'status'
        data = self.__request(url, post_param)

        if isinstance(data, dict):
            if data['status'] == 'OK' and data['status_code'] == 100:
                if sms_id in data['sms']:
                    return True, data['sms'][sms_id]
        return False, None

    def validate_callback(self, data: list, hash: str) -> bool:
        my_hash = self.__api_id + "".join(data)
        return hash == hashlib.sha256(my_hash.encode('utf-8')).hexdigest()

    def send_multi_sms(self, phone_sms: dict) -> dict:
        post_param = {
            "multi": dict()
        }
        for phone, msg in phone_sms.items():
            phone_beautify = self.beautify_phone(phone)
            if phone_beautify:
                post_param['multi'][phone_beautify] = msg

        return self._sms_request(post_param)

    def get_balance(self) -> float:
        url = self.__api_url_my + 'balance'
        data = self.__request(url, {})
        if data['status_code'] != 100:
            raise Exception(data['status_text'])

        return data['balance']

    def get_limit(self) -> dict:
        url = self.__api_url_my + 'limit'
        data = self.__request(url, {})
        if data['status_code'] != 100:
            raise Exception(data['status_text'])

        return {
            "total_limit": data['total_limit'],
            "used_today": data['used_today']
        }



# Generated by Django 3.1.6 on 2021-06-09 12:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('smsru', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='log',
            options={'ordering': ('-created_at',), 'verbose_name': 'Логи', 'verbose_name_plural': 'Лог'},
        ),
        migrations.AlterField(
            model_name='log',
            name='cost',
            field=models.FloatField(blank=True, null=True, verbose_name='Стоимость'),
        ),
        migrations.AlterField(
            model_name='log',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Дата создания'),
        ),
        migrations.AlterField(
            model_name='log',
            name='msg',
            field=models.TextField(verbose_name='Сообщение'),
        ),
        migrations.AlterField(
            model_name='log',
            name='phone',
            field=models.CharField(max_length=30, verbose_name='Номер телефона'),
        ),
        migrations.AlterField(
            model_name='log',
            name='status',
            field=models.CharField(max_length=10, verbose_name='Статус'),
        ),
        migrations.AlterField(
            model_name='log',
            name='status_code',
            field=models.IntegerField(verbose_name='Код статуса'),
        ),
        migrations.AlterField(
            model_name='log',
            name='status_text',
            field=models.TextField(blank=True, null=True, verbose_name='Текст статуса'),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-18 23:41

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('phone', models.CharField(max_length=30, verbose_name='Phone number')),
                ('msg', models.TextField(verbose_name='Message')),
                ('status', models.CharField(max_length=10, verbose_name='Status')),
                ('status_code', models.IntegerField(verbose_name='Status code')),
                ('status_text', models.TextField(blank=True, null=True, verbose_name='Status text')),
                ('sms_id', models.CharField(blank=True, max_length=255, null=True, verbose_name='SMS ID')),
                ('cost', models.FloatField(blank=True, null=True, verbose_name='Cost')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
            ],
            options={
                'verbose_name': 'Logs',
                'verbose_name_plural': 'Log',
                'ordering': ('-created_at',),
            },
        ),
    ]
